/*!
 * @file        bsp_cec.c
 *
 * @brief       HDMI CEC board support package program body
 *
 * @version     V1.0.0
 *
 * @date        2022-09-29
 *
 * @attention
 *
 *  Copyright (C) 2022 Geehy Semiconductor
 *
 *  You may not use this file except in compliance with the
 *  GEEHY COPYRIGHT NOTICE (GEEHY SOFTWARE PACKAGE LICENSE).
 *
 *  The program is only for reference, which is distributed in the hope
 *  that it will be usefull and instructional for customers to develop
 *  their software. Unless required by applicable law or agreed to in
 *  writing, the program is distributed on an "AS IS" BASIS, WITHOUT
 *  ANY WARRANTY OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the GEEHY SOFTWARE PACKAGE LICENSE for the governing permissions
 *  and limitations under the License.
 */

/* Includes */
#include "bsp_cec.h"
#include <stdio.h>

/** @addtogroup Board
  @{
*/

/** @addtogroup Board_APM32F072_EVAL
  @{
*/

/** @defgroup APM32F072_EVAL_Fuctions
  @{
  */

/*!
 * @brief       CEC initialization
 *
 * @param       initiatorAddr: CEC device initiator address
 *
 * @retval      CEC error code
 *
 */
CEC_ERROR_T CEC_Init(uint8_t initiatorAddr)
{
    CEC_ERROR_T errSta = CEC_OK;
    
    CEC_Config_T cecConfig;
    GPIO_Config_T gpioConfig;
    
    /* Enable Peripheral clock */
    RCM_EnableAPB1PeriphClock(RCM_APB1_PERIPH_CEC);
    
    /* Enable GPIO clock */
    RCM_EnableAHBPeriphClock(CEC_LINE_GPIO_CLK);
    RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_SYSCFG);
    
    gpioConfig.pin = CEC_LINE_PIN;
    gpioConfig.mode = GPIO_MODE_AF;
    gpioConfig.outtype = GPIO_OUT_TYPE_OD;
    gpioConfig.speed = GPIO_SPEED_50MHz;
    gpioConfig.pupd = GPIO_PUPD_NO;
    GPIO_Config(CEC_LINE_GPIO_PORT,&gpioConfig);
    
    /* Connect GPIO to CEC AF*/
    GPIO_ConfigPinAF(CEC_LINE_GPIO_PORT,CEC_LINE_SOURCE,CEC_LINE_GPIO_AF);
    
    /* Enable the CEC global Interrupt*/
    NVIC_EnableIRQRequest(CEC_CAN_IRQn, 0x01);
    
    /* Configure CEC */
    CEC_ConfigStructInit(&cecConfig);
    cecConfig.signalFreeTime            = CEC_SINGANL_FREETIME_STANDARD;
    cecConfig.RxTolerance               = CEC_RX_TOLERANCE_STANDARD;
    cecConfig.stopReception             = CEC_STOP_RECEPTION_OFF;
    cecConfig.bitRisingError            = CEC_BIT_RISING_ERR_OFF;
    cecConfig.longPeriodError           = CEC_LONG_PERIOD_ERR_OFF;
    cecConfig.broadcastrNoGen           = CEC_BROADCAST_NO_ERR_OFF;
    cecConfig.signalFreeTimeOption      = CEC_SIGNAL_FREETIME_OPTION_OFF;
    CEC_Config(&cecConfig);
    
    /* Set the CEC initiator address*/
    CEC_ConfigOwnAddress(initiatorAddr);
    
    /* Activate CEC interrupts associated to the set of TX flags */
    CEC_EnableInterrupt(CEC_INT_TXEND | CEC_INT_TXBR);
    
    /* Activate CEC interrupts associated to the set of RX flags */
    CEC_EnableInterrupt(CEC_INT_RXEND | CEC_INT_RXBR);
    
    /* Activate CEC interrupts associated to the set of TX error */
    CEC_EnableInterrupt(CEC_INT_TXACKE | CEC_INT_TXERR | CEC_INT_TXUDR | CEC_INT_ARBLST);
    
    /* Activate CEC interrupts associated to the set of RX error */
    CEC_EnableInterrupt(CEC_INT_RXACKE | CEC_INT_LBPE | CEC_INT_SBPE | CEC_INT_BRE | CEC_INT_RXOVR);
    
    CEC_Enable();
    
    return errSta;
}

/**@} end of group APM32F072_EVAL_Functions */
/**@} end of group Board_APM32F072_EVAL */
/**@} end of group Board */
