/*!
 * @file        main.c
 *
 * @brief       Main program body
 *
 * @version     V1.0.0
 *
 * @date        2022-09-29
 *
 * @attention
 *
 *  Copyright (C) 2022 Geehy Semiconductor
 *
 *  You may not use this file except in compliance with the
 *  GEEHY COPYRIGHT NOTICE (GEEHY SOFTWARE PACKAGE LICENSE).
 *
 *  The program is only for reference, which is distributed in the hope
 *  that it will be usefull and instructional for customers to develop
 *  their software. Unless required by applicable law or agreed to in
 *  writing, the program is distributed on an "AS IS" BASIS, WITHOUT
 *  ANY WARRANTY OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the GEEHY SOFTWARE PACKAGE LICENSE for the governing permissions
 *  and limitations under the License.
 */

/* Includes */
#include "main.h"
#include <stdio.h>
#include <string.h>
#include "bsp_lcd.h"
#include "bsp_key.h"

/** @addtogroup Examples
  * @brief CAN Two Boards examples
  @{
  */

/** @addtogroup CAN_TwoBoards
  @{
  */

/** @defgroup CAN_TwoBoards_Macros Macros
  @{
*/

/* printf using USART1  */
#define DEBUG_USART             USART1
#define CAN_STD_ID              0x3A
#define CAN_EXT_ID              0x01

/**@} end of group CAN_TwoBoards_Macros*/

/** @defgroup CAN_TwoBoards_Enumerates Enumerates
  @{
  */

/*!
 * @brief       CAN communication status
 */
typedef enum {
    CAN_OK,
    CAN_ERROR,
    CAN_BUSY,
} CAN_Status_T;

/**@} end of group CAN_TwoBoards_Enumerates*/

/** @defgroup CAN_TwoBoards_Variables Variables
  @{
  */

/* Timer tick */
uint16_t cntTick = 0;

__IO CAN_Status_T flag = CAN_ERROR;
CAN_Tx_Message TxMessage;
CAN_Rx_Message RxMessage;

/* Menu title string */
const char *menuTitleStr[] = {
    " CAN Two Boards Example ",
};

/* Menu item string */
const char *menuItemStr[] = {
    "Press KEY1 to start test",
    "       Test OK          ",
    "       Test Error       ",
    "       Testing          ",
    "                        ",
};

/* Menu bottom column string */
const char *menuBottomColStr[] = {
    "   GEEHY SEMICONDUCTOR   ",
};

/**@} end of group CAN_TwoBoards_Variables*/

/** @defgroup CAN_TwoBoards_Functions Functions
  @{
  */

void APM_EVAL_Init(void);
CAN_Status_T CAN_TwoBoardsTest(void);
CAN_Status_T CAN_Init(void);
void Menu_DisplayInit(void);
void APM_EVAL_TMR14_Init(uint16_t period, uint16_t div);

/*!
 * @brief       Main program
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
int main(void)
{
    APM_EVAL_Init();
    
    printf("This is an example of CAN two boards\r\n");

    /* Initialize CAN TX message*/
    TxMessage.stanID            = CAN_STD_ID;
    TxMessage.extenID           = CAN_EXT_ID;
    TxMessage.remoteTxReq       = CAN_RTXR_DATA;
    TxMessage.typeID            = CAN_TYPEID_STD;
    TxMessage.dataLengthCode    = 1;
    
    while(1)
    {
        CAN_TwoBoardsTest();
    }
}

/*!
 * @brief       Init EVAL board
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void APM_EVAL_Init(void)
{
    /* Init delay function */
    APM_EVAL_DelayInit();
    
    APM_EVAL_LEDInit(LED1);
    APM_EVAL_LEDInit(LED2);
    APM_EVAL_LEDInit(LED3);
    APM_EVAL_LEDInit(LED4);
    APM_EVAL_PBInit(BUTTON_KEY1,BUTTON_MODE_GPIO);
    APM_EVAL_COMInit(COM1);
    
    APM_EVAL_TMR14_Init(1000,48);
    APM_LCDInit();
    LCD_Clear(0,0,LCD_WIDTH,LCD_HEIGHT,LCD_BCOLOR);
    
    Menu_DisplayInit();
    
    CAN_Init();
}

/*!
 * @brief       CAN init
 *
 * @param       None
 *
 * @retval      status code
 *
 */
CAN_Status_T CAN_Init(void)
{
    CAN_Status_T errSta = CAN_OK;
    
    GPIO_Config_T       gpioConfigStruct;
    CAN_Config_T        canConfigStruct;
    CAN_FilterConfig_T  filterConfigStruct;
    
    /* Enable CAN clock */
    RCM_EnableAPB1PeriphClock(CAN_PERIPH_CLK);
    
    /* Enable CAN GPIO clock*/
    RCM_EnableAHBPeriphClock(CAN_TX_GPIO_CLK | CAN_RX_GPIO_CLK);
    
    GPIO_ConfigPinAF(CAN_TX_GPIO_PORT,CAN_TX_SOURCE,CAN_TX_AF_PORT);
    GPIO_ConfigPinAF(CAN_RX_GPIO_PORT,CAN_RX_SOURCE,CAN_RX_AF_PORT);
    
    /* Configure CAN pins */
    gpioConfigStruct.pin        = CAN_TX_PIN;
    gpioConfigStruct.mode       = GPIO_MODE_AF;
    gpioConfigStruct.speed      = GPIO_SPEED_50MHz;
    gpioConfigStruct.outtype    = GPIO_OUT_TYPE_PP;
    gpioConfigStruct.pupd       = GPIO_PUPD_PU;
    GPIO_Config(CAN_TX_GPIO_PORT,&gpioConfigStruct);
    
    gpioConfigStruct.pin = CAN_RX_PIN;
    GPIO_Config(CAN_TX_GPIO_PORT,&gpioConfigStruct);
    
    /* Configure CAN*/
    CAN_Reset();
    CAN_ConfigStructInit(&canConfigStruct);
    
    /* CAN configuration */
    CAN_ConfigStructInit(&canConfigStruct);
    canConfigStruct.autoBusOffManage = DISABLE;
    canConfigStruct.autoWakeUpMode   = DISABLE;
    canConfigStruct.nonAutoRetran    = DISABLE;
    canConfigStruct.timeTrigComMode  = DISABLE;
    canConfigStruct.txFIFOPriority   = DISABLE;
    canConfigStruct.rxFIFOLockMode   = DISABLE;
    canConfigStruct.mode             = CAN_MODE_NORMAL;
    canConfigStruct.syncJumpWidth    = CAN_SJW_1;
    
    /* CAN Baudrate = 1MBps (CAN clocked at 48 MHz) */
    canConfigStruct.timeSegment1     = CAN_TIME_SEGMENT1_15;
    canConfigStruct.timeSegment2     = CAN_TIME_SEGMENT2_8;
    canConfigStruct.prescaler        = 2;
    CAN_Config(&canConfigStruct);
    
    filterConfigStruct.filterNumber = CAN_FILTER_NUMBER_1;
    filterConfigStruct.filterMode   = CAN_FILTER_MODE_IDMASK;
    filterConfigStruct.filterScale  = CAN_FILTER_SCALE_32BIT;
    filterConfigStruct.filterIdHigh = 0x6420;
    filterConfigStruct.filterIdLow  = 0x0000;
    filterConfigStruct.filterMaskIdHigh = 0x0000;
    filterConfigStruct.filterMaskIdLow  = 0x0000;
    filterConfigStruct.filterFIFO       = CAN_FIFO_0;
    filterConfigStruct.filterActivation = ENABLE;
    CAN_ConfigFilter(&filterConfigStruct);
    
    CAN_EnableInterrupt(CAN_INT_F0MP);
    
    /* NVIC configuration */
    NVIC_EnableIRQRequest(CEC_CAN_IRQn,0);
    
    return errSta;
}

/*!
 * @brief       CAN loopback test
 *
 * @param       None
 *
 * @retval      status code
 *
 */
CAN_Status_T CAN_TwoBoardsTest(void)
{
    CAN_Status_T errSta = CAN_OK;
    static uint16_t temp = 0;
    char str[2];
    
    if(APM_EVAL_PBGetState(BUTTON_KEY1) == 0)
    {
        TxMessage.data[0] = temp;
        CAN_TxMessage(&TxMessage);

        temp++;
        if(temp > 0xFF)
        {
            temp = 0;
        }
        
        printf("CAN Send :0x%02X\r\n",TxMessage.data[0]);
        while(APM_EVAL_PBGetState(BUTTON_KEY1) == 0);
    }
    
    /* CAN send and receive UI*/
    sprintf(str,"%02X",TxMessage.stanID);
    LCD_DisplayString(70,LCD_LINE_1, str,RGB2RGB565(255,0,22),LCD_TEXT_BC,16,0);
    
    sprintf(str,"%02X",TxMessage.extenID);
    LCD_DisplayString(70,LCD_LINE_2, str,RGB2RGB565(255,0,22),LCD_TEXT_BC,16,0);
    
    sprintf(str,"%02X",TxMessage.dataLengthCode);
    LCD_DisplayString(70,LCD_LINE_3, str,RGB2RGB565(255,0,22),LCD_TEXT_BC,16,0);
    
    sprintf(str,"%02X",TxMessage.data[0]);
    LCD_DisplayString(70,LCD_LINE_4, str,RGB2RGB565(255,0,22),LCD_TEXT_BC,16,0);
    
    
    sprintf(str,"%02X",RxMessage.stanID);
    LCD_DisplayString(230,LCD_LINE_1, str,RGB2RGB565(255,0,22),LCD_TEXT_BC,16,0);
    
    sprintf(str,"%02X",RxMessage.extenID);
    LCD_DisplayString(230,LCD_LINE_2, str,RGB2RGB565(255,0,22),LCD_TEXT_BC,16,0);
    
    sprintf(str,"%02X",RxMessage.dataLengthCode);
    LCD_DisplayString(230,LCD_LINE_3, str,RGB2RGB565(255,0,22),LCD_TEXT_BC,16,0);
    
    sprintf(str,"%02X",RxMessage.data[0]);
    LCD_DisplayString(230,LCD_LINE_4, str,RGB2RGB565(255,0,22),LCD_TEXT_BC,16,0);
    
    return errSta;
}

/*!
 * @brief       CAN IRQ callback
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void CAN_Isr(void)
{
    CAN_RxMessage(CAN_FIFO_0, &RxMessage);

    if ((RxMessage.stanID == CAN_STD_ID) && (RxMessage.typeID == CAN_TYPEID_STD))
    {
        printf("CAN Receive:0x%02X !\r\n",RxMessage.data[0]);
    }
    else
    {
        printf("CAN Receive Error !\r\n");
    }
}

/*!
 * @brief       Initialize the Menu UI
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void Menu_DisplayInit(void)
{
    /* Menu title*/
    LCD_Clear(0,0,LCD_WIDTH,40,LCD_FCOLOR);
    LCD_DisplayString(10,LCD_LINE_TITLE,menuTitleStr[0],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
    
    /* Menu content*/
    /* CAN send and receive UI*/
    LCD_DisplayString(30,LCD_LINE_0, "  Send  ",LCD_TEXT_FC,LCD_TEXT_BC,24,0);
    LCD_DisplayString(200,LCD_LINE_0,"Receive",LCD_TEXT_FC,LCD_TEXT_BC,24,0);
    LCD_DrawLine(160,50,160,210,LCD_FCOLOR);
    LCD_DrawLine(10,76,150,76,LCD_FCOLOR);
    LCD_DrawLine(170,76,310,76,LCD_FCOLOR);
    
    LCD_DisplayString(10,LCD_LINE_1, "StdID:",LCD_TEXT_FC,LCD_TEXT_BC,16,0);
    
    LCD_DisplayString(10,LCD_LINE_2, "ExtID:",LCD_TEXT_FC,LCD_TEXT_BC,16,0);
    
    LCD_DisplayString(10,LCD_LINE_3, "LEN  :",LCD_TEXT_FC,LCD_TEXT_BC,16,0);
    
    LCD_DisplayString(10,LCD_LINE_4, "Data :",LCD_TEXT_FC,LCD_TEXT_BC,16,0);
    
    LCD_DisplayString(10,LCD_LINE_5, "Press KEY1 to send",LCD_TEXT_FC,LCD_TEXT_BC,16,0);
    
    LCD_DisplayString(170,LCD_LINE_1, "StdID:",LCD_TEXT_FC,LCD_TEXT_BC,16,0);
    
    LCD_DisplayString(170,LCD_LINE_2, "ExtID:",LCD_TEXT_FC,LCD_TEXT_BC,16,0);
    
    LCD_DisplayString(170,LCD_LINE_3, "LEN  :",LCD_TEXT_FC,LCD_TEXT_BC,16,0);
    
    LCD_DisplayString(170,LCD_LINE_4, "Data :",LCD_TEXT_FC,LCD_TEXT_BC,16,0);
    
    LCD_DisplayString(170,LCD_LINE_5, "   Auto receive",LCD_TEXT_FC,LCD_TEXT_BC,16,0);
    
    /* Menu bottom*/
    LCD_Clear(0,LCD_LINE_BOTTOM,LCD_WIDTH,LCD_HEIGHT,LCD_FCOLOR);
    LCD_DisplayString(60,LCD_LINE_BOTTOM,menuBottomColStr[0],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,16,0);
}

/*!
 * @brief       TMR14 init
 *
 * @param       None
 *
 * @retval      None
 *
 */
void APM_EVAL_TMR14_Init(uint16_t period, uint16_t div)
{
    TMR_TimeBase_T  timeBaseConfig;

    /* Enable Clock*/
    RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_SYSCFG);
    RCM_EnableAPB1PeriphClock(RCM_APB1_PERIPH_TMR14);

    /* Set clockDivision = 1 */
    timeBaseConfig.clockDivision =  TMR_CKD_DIV1;
    /* Up-counter */
    timeBaseConfig.counterMode =  TMR_COUNTER_MODE_UP;
    /* Set divider */
    timeBaseConfig.div = div - 1 ;
    /* Set counter = 0xffff */
    timeBaseConfig.period = period - 1;
    /* Repetition counter = 0x0 */
    timeBaseConfig.repetitionCounter =  0;

    TMR_ConfigTimeBase(TMR14,&timeBaseConfig);

    /* Enable update interrupt*/
    TMR_EnableInterrupt(TMR14,TMR_INT_UPDATE);
    NVIC_EnableIRQRequest(TMR14_IRQn,2);

    /*  Enable TMR14  */
    TMR_Enable(TMR14);
}

/*!
 * @brief       TMR4 interrupt service runtine
 *
 * @param       None
 *
 * @retval      None
 *
 * @note        This function need to put into TMR14_IRQHandler() in apm32f0xx_int.c
 */
void TMR14_Isr(void)
{
    if(TMR_ReadIntFlag(TMR14,TMR_INT_FLAG_UPDATE) == SET)
    {
        TMR_ClearIntFlag(TMR14,TMR_INT_FLAG_UPDATE);
        cntTick++;
        
        if(cntTick >= 500)
        {
            cntTick = 0;
            APM_EVAL_LEDToggle(LED1);
        }
    }
}

/*!
 * @brief       Redirect C Library function printf to serial port.
 *              After Redirection, you can use printf function.
 *
 * @param       ch:  The characters that need to be send.
 *
 * @param       *f:  pointer to a FILE that can recording all information
 *              needed to control a stream
 *
 * @retval      The characters that need to be send.
 *
 * @note
 */
int fputc(int ch, FILE *f)
{
    /** send a byte of data to the serial port */
    USART_TxData(DEBUG_USART,(uint8_t)ch);

    /** wait for the data to be send  */
    while (USART_ReadStatusFlag(DEBUG_USART, USART_FLAG_TXBE) == RESET);

    return (ch);
}

/**@} end of group CAN_TwoBoards_Functions */
/**@} end of group CAN_TwoBoards */
/**@} end of group Examples */
