/*!
 * @file        main.c
 *
 * @brief       Main program body
 *
 * @version     V1.0.0
 *
 * @date        2022-09-29
 *
 * @attention
 *
 *  Copyright (C) 2022 Geehy Semiconductor
 *
 *  You may not use this file except in compliance with the
 *  GEEHY COPYRIGHT NOTICE (GEEHY SOFTWARE PACKAGE LICENSE).
 *
 *  The program is only for reference, which is distributed in the hope
 *  that it will be usefull and instructional for customers to develop
 *  their software. Unless required by applicable law or agreed to in
 *  writing, the program is distributed on an "AS IS" BASIS, WITHOUT
 *  ANY WARRANTY OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the GEEHY SOFTWARE PACKAGE LICENSE for the governing permissions
 *  and limitations under the License.
 */

/* Includes */
#include "main.h"
#include "bsp_delay.h"
#include "bsp_key.h"
#include "bsp_lcd.h"
#include "bsp_cec.h"
#include <stdio.h>

/** @addtogroup Examples
  * @brief CEC two boards data exchange examples
  @{
  */

/** @addtogroup CEC_TwoBoardsExchange
  @{
  */

/** @defgroup CEC_TwoBoardsExchange_Macros Macros
  @{
*/
#define DEBUG_USART             USART1
#define CEC_USR_CMD_MAX         5

/**@} end of group CEC_TwoBoardsExchange_Macros*/

/** @defgroup CEC_TwoBoardsExchange_Enumerates Enumerates
  @{
  */

/**
 * @brief   CEC two boards device types enumeration
 */
typedef enum
{
    CEC_DEV_1 = 1,
    CEC_DEV_2,
    CEC_DEV_UNREGISTERED,
} CEC_TWO_BOARD_DEVICE_T;

/**
 * @brief   CEC user control command
 */
typedef enum
{
    CEC_USR_VOL_UP,
    CEC_USR_VOL_DOWN,
    CEC_USR_MUTE,
    CEC_USR_PLAY,
    CEC_USR_STOP,
} CEC_USR_CMD_T;

/**@} end of group CEC_TwoBoardsExchange_Enumerates*/

/** @defgroup CEC_TwoBoardsExchange_Variables Variables
  @{
  */

/* Timer tick */
uint16_t cntTick = 0;

CEC_INFO_T cecInfo;
CEC_MSG_TRANS_T cecTransMsg;
CEC_MSG_PACK_T cecRxMsg;
CEC_MSG_PACK_T cecTxMsg;

/* Menu communication status*/
const char *menuCommStaStr[] = {
    " Succeeded ",
    "   Failed  ",
    " Conflicted",
};

/* Menu CEC device commands*/
const char *menuCECDevCmdStr[] = {
    "<       VOLUME UP       >",
    "<      VOLUME DOWN      >",
    "<          MUTE         >",
    "<          PLAY         >",
    "<          STOP         >",
};

/* Menu title string*/
const char *menuTitleStr[] = {
    " Select CEC Device Type  ",
    "     Initializing...     ",
    "       Initialized       ",
    "      Unregistered !!!   ",
    "   Communication Error   ",
};

/* Menu item string*/
const char *menuItemStr[] = {
    " 1.CEC Device 01         ",
    " 2.CEC Device 02         ",
};

/* Menu bottom column string*/
const char *menuBottomColStr[] = {
    "   GEEHY SEMICONDUCTOR   ",
    "       CEC DEV 01        ",
    "       CEC DEV 02        ",
};

uint8_t curFuncIndex = 0;
uint8_t menuChange = 0;

/**@} end of group CEC_TwoBoardsExchange_Variables*/

/** @defgroup CEC_TwoBoardsExchange_Functions Functions
  @{
  */

/* CEC Function*/
void CEC_ControlHandle(uint8_t menuState);
void CEC_HomePageHandle(uint8_t menuState);
void CEC_MenuHandle(void);
void CEC_Handle(uint8_t menuState);
void CEC_DisplayInit(void);
void CEC_RevInfoDisplay(void);
void CEC_DevAddrConfig(uint8_t devType);
CEC_ERROR_T CEC_CmdCallback(uint8_t opcode,uint8_t cmdIndex);

void APM_EVAL_Init(void);
void APM_EVAL_TMR14_Init(uint16_t period, uint16_t div);

void (*curFunction)(uint8_t param);

/**@} end of group CEC_TwoBoardsExchange_Functions */

/** @defgroup CEC_TwoBoardsExchange_Variables Variables
  @{
  */

/* Menu table*/
MENU_INFO_T CEC_MenuTable[] = {
    /* current menu, enter, prev, next, return, function*/
    {0,0,2,1,0,(*CEC_HomePageHandle),0},
    {1,3,2,2,0,(*CEC_HomePageHandle),1},
    {2,4,1,1,0,(*CEC_HomePageHandle),2},
    
    {3,5,3,3,0,(*CEC_Handle),CEC_DEV_1},
    {4,5,4,4,0,(*CEC_Handle),CEC_DEV_2},
    
    {5,5,5,5,0,(*CEC_ControlHandle),0},
};

/**@} end of group CEC_TwoBoardsExchange_Variables*/

/** @defgroup CEC_TwoBoardsExchange_Functions Functions
  @{
  */

/*!
 * @brief       Main program
 *
 * @param       None
 *
 * @retval      None
 *
 */
int main(void)
{
    APM_EVAL_Init();
    
    while(1)
    {
        CEC_MenuHandle();
    }
}

/*!
 * @brief       Init EVAL board
 *
 * @param       None
 *
 * @retval      None
 *
 */
void APM_EVAL_Init(void)
{
    APM_EVAL_LEDInit(LED1);
    APM_EVAL_LEDInit(LED2);
    APM_EVAL_LEDInit(LED3);
    APM_EVAL_LEDInit(LED4);
    
    APM_EVAL_COMInit(COM1);
    APM_EVAL_LEDOn(LED1);
    APM_EVAL_LEDOn(LED2);
    APM_EVAL_LEDOn(LED3);
    APM_EVAL_LEDOn(LED4);
    
    APM_EVAL_DelayInit();
    APM_EVAL_TMR14_Init(1000,48);
    
    APM_LCDInit();
    
    LCD_Clear(0,0,LCD_WIDTH,LCD_HEIGHT,LCD_BCOLOR);
    
    CEC_DisplayInit();
}



/*!
 * @brief       De-Initialize the CEC
 *
 * @param       None
 *
 * @retval      None
 *
 */
void CEC_DeInit(void)
{
    CEC_Config_T cecConfig;
    
    CEC_ConfigStructInit(&cecConfig);
}

/*!
 * @brief       Config CEC device address
 *
 * @param       devType: device type
 *
 * @retval      None
 *
 */
void CEC_DevAddrConfig(uint8_t devType)
{
    /* Configure CEC initiator and destination address*/
    if(devType == CEC_DEV_1)
    {
        cecInfo.initiatorAddr      = CEC_DEV_1;
        cecInfo.destinationAddr    = CEC_DEV_2;
    }
    else if(devType == CEC_DEV_2)
    {
        cecInfo.initiatorAddr      = CEC_DEV_2;
        cecInfo.destinationAddr    = CEC_DEV_1;
    }
    else
    {
        cecInfo.initiatorAddr      = CEC_DEV_UNREGISTERED;
        cecInfo.destinationAddr    = CEC_DEV_UNREGISTERED;
    }
}

/*!
 * @brief       Initialize the CEC UI
 *
 * @param       None
 *
 * @retval      None
 *
 */
void CEC_DisplayInit(void)
{
    /* Menu title*/
    LCD_Clear(0,0,LCD_WIDTH,40,LCD_FCOLOR);
    LCD_DisplayString(10,LCD_LINE_TITLE,menuTitleStr[0],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
    
    /* Menu item*/
    LCD_DisplayString(10,LCD_LINE_0,menuItemStr[0],LCD_TEXT_FC,LCD_TEXT_BC,24,0);
    LCD_DisplayString(10,LCD_LINE_1,menuItemStr[1],LCD_TEXT_FC,LCD_TEXT_BC,24,0);
    
    /* Menu bottom*/
    LCD_Clear(0,LCD_LINE_BOTTOM,LCD_WIDTH,LCD_HEIGHT,LCD_FCOLOR);
    LCD_DisplayString(60,LCD_LINE_BOTTOM,menuBottomColStr[0],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,16,0);
}

/*!
 * @brief       Initialize the CEC home page UI
 *
 * @param       None
 *
 * @retval      None
 *
 */
void CEC_HomePageHandle(uint8_t menuState)
{
    switch(menuState)
    {
        case 0:
            /* Clear content*/
            LCD_Clear(0,40,LCD_WIDTH,200,LCD_BCOLOR);
            LCD_Clear(0,0,LCD_WIDTH,40,LCD_FCOLOR);
            /* Menu title*/
            LCD_Clear(0,0,LCD_WIDTH,40,LCD_FCOLOR);
            LCD_DisplayString(10,LCD_LINE_TITLE,menuTitleStr[0],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
            
            /* Menu item*/
            LCD_DisplayString(10,LCD_LINE_0,menuItemStr[0],LCD_TEXT_FC,LCD_TEXT_BC,24,0);
            LCD_DisplayString(10,LCD_LINE_1,menuItemStr[1],LCD_TEXT_FC,LCD_TEXT_BC,24,0);
        
            /* Menu bottom*/
            LCD_Clear(0,LCD_LINE_BOTTOM,LCD_WIDTH,LCD_HEIGHT,LCD_FCOLOR);
            LCD_DisplayString(60,LCD_LINE_BOTTOM,menuBottomColStr[0],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,16,0);
            break;

        case 1:
            /* Menu item*/
            LCD_DisplayString(10,LCD_LINE_1,menuItemStr[1],LCD_TEXT_FC,LCD_TEXT_BC,24,0);
            LCD_DisplayString(10,LCD_LINE_0,menuItemStr[0],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
            break;
        
        case 2:
            /* Menu item*/
            LCD_DisplayString(10,LCD_LINE_0,menuItemStr[0],LCD_TEXT_FC,LCD_TEXT_BC,24,0);
            LCD_DisplayString(10,LCD_LINE_1,menuItemStr[1],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
            break;
    }
}

/*!
 * @brief       CEC controller handle
 *
 * @param       None
 *
 * @retval      None
 *
 */
void CEC_ControlHandle(uint8_t menuState)
{
    uint8_t keyType;
    int8_t cmdIndex = 0;
    
    /* Clear content*/
    LCD_Clear(0,40,LCD_WIDTH,200,LCD_BCOLOR);
    LCD_Clear(0,0,LCD_WIDTH,40,LCD_FCOLOR);
    
    LCD_DisplayString(10,LCD_LINE_TITLE,"Receive:",LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
    
    LCD_DrawLine(10,150,310,150,LCD_FCOLOR);
    
    LCD_Clear(0,165,LCD_WIDTH,200,LCD_FCOLOR);
    LCD_DisplayString(10,LCD_LINE_4,"Send   :",LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
    LCD_DisplayString(110,176,menuCECDevCmdStr[cmdIndex],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,16,0);
    
    /* Loop */
    while(KEY4)
    {
        keyType = Key_Scan(KEY_MODE_SINGLE);
        
        switch(keyType)
        {
            case MENU_KEY_PREV:
                if(cmdIndex == 0)
                {
                    cmdIndex = CEC_USR_CMD_MAX;
                }
                cmdIndex--;
                
                LCD_DisplayString(110,176,menuCECDevCmdStr[cmdIndex],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,16,0);
                break;
            
            case MENU_KEY_NEXT:
                if(cmdIndex == (CEC_USR_CMD_MAX - 1))
                {
                    cmdIndex = -1;
                }
                cmdIndex++;
                
                LCD_DisplayString(110,176,menuCECDevCmdStr[cmdIndex],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,16,0);
                break;
            
            case MENU_KEY_ENTER:
                /* Send CEC message*/
                CEC_CmdCallback(CEC_USER_CONTROL_PRESSED_OPCODE,cmdIndex);
                break;
        }
        
        /* Display the received messages */
        CEC_RevInfoDisplay();
    }
    
    CEC_DeInit();
    curFuncIndex = CEC_MenuTable[curFuncIndex].retIndex;
    return;
}

/*!
 * @brief       CEC handle
 *
 * @param       None
 *
 * @retval      None
 *
 */
void CEC_Handle(uint8_t menuState)
{
    uint8_t errSta = 0;
    
    /* Clear content*/
    LCD_Clear(0,40,LCD_WIDTH,200,LCD_BCOLOR);
    LCD_Clear(0,0,LCD_WIDTH,40,LCD_FCOLOR);
    
    /* Menu title*/
    LCD_DisplayString(10,LCD_LINE_TITLE,menuTitleStr[1],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
    
    switch(menuState)
    {
        case CEC_DEV_1:
            cecInfo.deviceType = CEC_DEV_1;
            CEC_DevAddrConfig(cecInfo.deviceType);
            /* Menu bottom*/
            LCD_DisplayString(60,LCD_LINE_BOTTOM,menuBottomColStr[1],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,16,0);
            break;
        
        case CEC_DEV_2:
            cecInfo.deviceType = CEC_DEV_2;
            CEC_DevAddrConfig(cecInfo.deviceType);
            /* Menu bottom*/
            LCD_DisplayString(60,LCD_LINE_BOTTOM,menuBottomColStr[2],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,16,0);
            break;
        
        default:
            cecInfo.deviceType = CEC_DEV_UNREGISTERED;
            CEC_DevAddrConfig(cecInfo.deviceType);
            break;
    }
    
    /* Init CEC */
    errSta = CEC_Init(cecInfo.initiatorAddr);
    cecTransMsg.rxCnt = 0;
    cecTransMsg.rxFrame = 0;
    
    cecTransMsg.rxStatus = CEC_TRANSMIT_BUSY;
    
    if(errSta == CEC_OK)
    {
        /* For Unregistered Device */
        if(cecInfo.initiatorAddr == CEC_DEV_UNREGISTERED)
        {
            /* Clear content*/
            LCD_Clear(0,40,LCD_WIDTH,200,LCD_BCOLOR);
            LCD_Clear(0,0,LCD_WIDTH,40,LCD_FCOLOR);
            
            /* Menu title*/
            LCD_DisplayString(10,LCD_LINE_TITLE,menuTitleStr[3],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
            
            LCD_DisplayString(10,LCD_LINE_2,"  Please press return.   ",LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
            
            /* Watting return press*/
            while(KEY4);
            CEC_DeInit();
            curFuncIndex = CEC_MenuTable[curFuncIndex].retIndex;
            return;
        }
        else
        {
            /* Display initialization */
            /* Menu title*/
            LCD_DisplayString(10,LCD_LINE_TITLE,menuTitleStr[2],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
            
            LCD_DisplayString(10,LCD_LINE_0,"Initiator Address   :    ",LCD_FCOLOR,LCD_BCOLOR,16,0);
            LCD_DisplayIntNum(180,LCD_LINE_0,cecInfo.initiatorAddr,2,RGB2RGB565(255,0,22),LCD_BCOLOR,16);
            
            LCD_DisplayString(10,LCD_LINE_1,"Destination Address :    ",LCD_FCOLOR,LCD_BCOLOR,16,0);
            LCD_DisplayIntNum(180,LCD_LINE_1,cecInfo.destinationAddr,2,RGB2RGB565(255,0,22),LCD_BCOLOR,16);
            
            LCD_DisplayString(10,LCD_LINE_3,"Please press ENTER to goto CEC OP menu",LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,16,0);
            
            /* Watting return or enter press*/
            while((KEY3) && (KEY4));
            if(KEY4 == 0)
            {
                CEC_DeInit();
                curFuncIndex = CEC_MenuTable[curFuncIndex].retIndex;
                return;
            }
            else if(KEY3 == 0)
            {
                curFuncIndex = CEC_MenuTable[curFuncIndex].enterIndex;
                return;
            }
        }
    }
    else
    {
        /* Clear content*/
        LCD_Clear(0,40,LCD_WIDTH,200,LCD_BCOLOR);
        LCD_Clear(0,0,LCD_WIDTH,40,LCD_FCOLOR);
        
        /* Menu title*/
        LCD_DisplayString(10,LCD_LINE_TITLE,menuTitleStr[4],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
        
        LCD_DisplayString(10,LCD_LINE_2,"  Please press return.   ",LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
        
        /* Watting return press*/
        while(KEY4);
        CEC_DeInit();
        curFuncIndex = CEC_MenuTable[curFuncIndex].retIndex;
        return;
    }
}

/*!
 * @brief       CEC menu handle
 *
 * @param       None
 *
 * @retval      None
 *
 */
void CEC_MenuHandle(void)
{
    uint8_t keyType;
    static uint8_t menuTemp = 0;
    
    keyType = Key_Scan(KEY_MODE_SINGLE);
    
    switch(keyType)
    {
        case MENU_KEY_PREV:
            curFuncIndex = CEC_MenuTable[curFuncIndex].prevIndex;
            break;
        
        case MENU_KEY_NEXT:
            curFuncIndex = CEC_MenuTable[curFuncIndex].nextIndex;
            break;
        
        case MENU_KEY_ENTER:
            curFuncIndex = CEC_MenuTable[curFuncIndex].enterIndex;
            break;
        
        case MENU_KEY_RETURN:
            curFuncIndex = CEC_MenuTable[curFuncIndex].retIndex;
            break;
    }
    
    if(menuTemp !=  CEC_MenuTable[curFuncIndex].curMenu)
    {
        menuTemp = CEC_MenuTable[curFuncIndex].curMenu;
        
        (*CEC_MenuTable[curFuncIndex].function)(CEC_MenuTable[curFuncIndex].stateMenu);
    }
}

/*!
 * @brief       CEC display receive data
 *
 * @param       buffer
 *
 * @retval      None
 *
 */
void CEC_RevDataDisplay(uint8_t *buffer,uint8_t size,uint16_t line)
{
    uint8_t i;
    uint16_t disLine = line;
    uint16_t disColumn;
    uint16_t columnStart = 150;
    char str[2];

    LCD_DisplayString(10,LCD_LINE_S_3,"Message Operand :",LCD_FCOLOR,LCD_BCOLOR,16,0);
    
    for(i = 0; i < size; i++)
    {
        if(i == 7)
        {
            disLine += 20;
            columnStart = 10;
            disColumn = 0;
        }
        
        if(i < 7)
        {
            disColumn = columnStart + i * 24;
        }
        else
        {
            disColumn = columnStart + (i - 7) * 24;
        }
        
        sprintf(str,"%02X",buffer[i]);
        LCD_DisplayString(disColumn,disLine,str,LCD_FCOLOR,LCD_BCOLOR,16,0);
    }
}

/*!
 * @brief       CEC display receive information
 *
 * @param       None
 *
 * @retval      None
 *
 */
void CEC_RevInfoDisplay(void)
{
    uint8_t i;
    char str[2];
    
    if(cecTransMsg.rxFrame)
    {
        /* Check if the frame has been received correctly */
        if(cecTransMsg.rxStatus == CEC_OK)
        {
            LCD_DisplayString(180,LCD_LINE_TITLE,menuCommStaStr[0],RGB2RGB565(8,255,87),LCD_BCOLOR,24,0);
        }
        else
        {
            LCD_DisplayString(180,LCD_LINE_TITLE,menuCommStaStr[1],RGB2RGB565(255,8,53),LCD_BCOLOR,24,0);
        }
        
        for(i = 0; i < cecTransMsg.rxCnt; i++)
        {
            printf("cecTransMsg.buffer:%d\r\n",cecTransMsg.buffer[i]);
        }
        
        cecRxMsg.rxMsgLen = cecTransMsg.rxCnt;
        
        cecRxMsg.header = cecTransMsg.buffer[0];
        cecRxMsg.opcode = cecTransMsg.buffer[1];
        for(i = 0;i < cecRxMsg.rxMsgLen - 2; i++)
        {
            cecRxMsg.operand[i] = cecTransMsg.buffer[i + 2];
        }
        
        /* Display the receive status, the initiator address, the received dat 
        frame and  the number of received bytes  */
        LCD_DisplayString(10,LCD_LINE_S_0,"Sender Address  :",LCD_FCOLOR,LCD_BCOLOR,16,0);
        
        sprintf(str,"%02X",cecRxMsg.header);
        LCD_DisplayString(150,LCD_LINE_S_0,str,LCD_FCOLOR,LCD_BCOLOR,16,0);
        
        LCD_DisplayString(10,LCD_LINE_S_1,"Number of Bytes :",LCD_FCOLOR,LCD_BCOLOR,16,0);
        
        sprintf(str,"%02X",(cecRxMsg.rxMsgLen + 2));
        LCD_DisplayString(150,LCD_LINE_S_1,str,LCD_FCOLOR,LCD_BCOLOR,16,0);
        
        LCD_DisplayString(10,LCD_LINE_S_2,"Message Opcode  :",LCD_FCOLOR,LCD_BCOLOR,16,0);
        
        sprintf(str,"%02X",cecRxMsg.opcode);
        LCD_DisplayString(150,LCD_LINE_S_2,str,LCD_FCOLOR,LCD_BCOLOR,16,0);
        CEC_RevDataDisplay((uint8_t*)cecRxMsg.operand,cecRxMsg.rxMsgLen - 2,LCD_LINE_S_3);
        
        /* Clear flag and buffer*/
        for(i = 0; i < cecTransMsg.rxCnt; i++)
        {
            cecTransMsg.buffer[i] = 0;
        }
        
        cecTransMsg.rxFrame = 0;
        cecTransMsg.rxCnt = 0;
    }
}

/*!
 * @brief       CEC command callback
 *
 * @param       opcode
 *
 * @param       cmdIndex
 *
 * @retval      CEC error code
 *
 * @note
 */
CEC_ERROR_T CEC_CmdCallback(uint8_t opcode,uint8_t cmdIndex)
{
    CEC_ERROR_T errSta = CEC_OK;
    CEC_MSG_PACK_T cecTxMsg;
    
    cecTransMsg.txCnt = 0;
    
    switch(opcode)
    {
        case CEC_USER_CONTROL_PRESSED_OPCODE:
            switch(cmdIndex)
            {
                case CEC_USR_VOL_UP:
                    printf("CEC_USR_VOL_UP\r\n");
                    
                    cecTxMsg.header = (((cecInfo.initiatorAddr & 0x0F) << 4) | (cecInfo.destinationAddr & 0x0F));
                    cecTxMsg.opcode = CEC_USER_CONTROL_PRESSED_OPCODE;
                    cecTxMsg.operand[0] = USR_OP_VOLUME_UP;
                    cecTxMsg.txMsgLen = 0x02;
                
                    cecTransMsg.buffer[cecTransMsg.txCnt++] = cecTxMsg.opcode;
                    cecTransMsg.buffer[cecTransMsg.txCnt++] = cecTxMsg.operand[0];
                    break;
                
                case CEC_USR_VOL_DOWN:
                    printf("CEC_USR_VOL_DOWN\r\n");
                
                    cecTxMsg.header = (((cecInfo.initiatorAddr & 0x0F) << 4) | (cecInfo.destinationAddr & 0x0F));
                    cecTxMsg.opcode = CEC_USER_CONTROL_PRESSED_OPCODE;
                    cecTxMsg.operand[0] = USR_OP_VOLUME_DOWN;
                    cecTxMsg.txMsgLen = 0x02;
                
                    cecTransMsg.buffer[cecTransMsg.txCnt++] = cecTxMsg.opcode;
                    cecTransMsg.buffer[cecTransMsg.txCnt++] = cecTxMsg.operand[0];
                    break;
                
                case CEC_USR_MUTE:
                    printf("CEC_USR_MUTE\r\n");
                
                    cecTxMsg.header = (((cecInfo.initiatorAddr & 0x0F) << 4) | (cecInfo.destinationAddr & 0x0F));
                    cecTxMsg.opcode = CEC_USER_CONTROL_PRESSED_OPCODE;
                    cecTxMsg.operand[0] = USR_OP_MUTE;
                    cecTxMsg.txMsgLen = 0x02;
                
                    cecTransMsg.buffer[cecTransMsg.txCnt++] = cecTxMsg.opcode;
                    cecTransMsg.buffer[cecTransMsg.txCnt++] = cecTxMsg.operand[0];
                    break;
                
                case CEC_USR_PLAY:
                    printf("CEC_USR_PLAY\r\n");
                
                    cecTxMsg.header = (((cecInfo.initiatorAddr & 0x0F) << 4) | (cecInfo.destinationAddr & 0x0F));
                    cecTxMsg.opcode = CEC_USER_CONTROL_PRESSED_OPCODE;
                    cecTxMsg.operand[0] = USR_OP_PLAY;
                    cecTxMsg.txMsgLen = 0x02;
                
                    cecTransMsg.buffer[cecTransMsg.txCnt++] = cecTxMsg.opcode;
                    cecTransMsg.buffer[cecTransMsg.txCnt++] = cecTxMsg.operand[0];
                    break;
                
                case CEC_USR_STOP:
                    printf("CEC_USR_STOP\r\n");
                
                    cecTxMsg.header = (((cecInfo.initiatorAddr & 0x0F) << 4) | (cecInfo.destinationAddr & 0x0F));
                    cecTxMsg.opcode = CEC_USER_CONTROL_PRESSED_OPCODE;
                    cecTxMsg.operand[0] = USR_OP_STOP;
                    cecTxMsg.txMsgLen = 0x02;
                
                    cecTransMsg.buffer[cecTransMsg.txCnt++] = cecTxMsg.opcode;
                    cecTransMsg.buffer[cecTransMsg.txCnt++] = cecTxMsg.operand[0];
                    break;
            }
            break;
    }
    
    cecTransMsg.txCnt = 0;
    CEC_TxData(cecTxMsg.header);

    /* Start new message*/
    CEC_StartNewMessage();
    
    return errSta;
}

/*!
 * @brief       CEC interrupts process function
 *
 * @param       None
 *
 * @retval      None
 *
 */
void CEC_Isr(void)
{
    /* Receive data*/
    /* Happen error*/
    if(CEC->STS & (CEC_INT_RXACKE | CEC_INT_LBPE | CEC_INT_SBPE | CEC_INT_BRE | CEC_INT_RXOVR))
    {
        CEC->STS = (CEC_INT_RXACKE | CEC_INT_LBPE | CEC_INT_SBPE | CEC_INT_BRE | CEC_INT_RXOVR);
        
        cecTransMsg.rxStatus = CEC_TRANSMIT_ERROR;
        cecTransMsg.rxFrame = 2;
    }
    
    if(CEC_ReadIntFlag(CEC_INT_RXBR))
    {
        /* Check received package is a header*/
        if(cecTransMsg.rxCnt == 0)
        {
            cecTransMsg.buffer[cecTransMsg.rxCnt] = ((CEC_RxData() >> 4) & 0x0F);
            cecTransMsg.rxCnt++;
        }
        /* Receive each byte except header and opcode in the reception buffer */
        else
        {
            cecTransMsg.buffer[cecTransMsg.rxCnt] = CEC_RxData();
            cecTransMsg.rxCnt++;
        }
        
        CEC_ClearIntFlag(CEC_INT_RXBR);
    }
    
    /* Check if the byte received is the last one of the message */
    if(CEC_ReadIntFlag(CEC_INT_RXEND))
    {
        cecTransMsg.rxStatus = CEC_OK;
        cecTransMsg.rxFrame = 1;
        
        /* Clear all reception flags */
        CEC_ClearIntFlag(CEC_INT_RXEND);
    }
    
    /* Send data*/
    
    /* Happen error*/
    if(CEC->STS & (CEC_INT_TXACKE | CEC_INT_TXERR | CEC_INT_TXUDR | CEC_INT_ARBLST))
    {
        CEC->STS = (CEC_INT_TXACKE | CEC_INT_TXERR | CEC_INT_TXUDR | CEC_INT_ARBLST);
        
        cecTransMsg.txStatus = CEC_TRANSMIT_ERROR;
    }
    
    /* Check if end of message bit is set in the data to be transmitted */
    if(CEC_ReadIntFlag(CEC_INT_TXEND))
    {
        /* Clear all reception flags */
        CEC_ClearIntFlag(CEC_INT_TXEND | CEC_INT_TXBR);
        
        cecTransMsg.txStatus = CEC_OK;
    }
    /* Check if data byte has been sent */
    else if(CEC_ReadIntFlag(CEC_INT_TXBR))
    {
        /* Set EOM bit if the byte to be transmitted is the last one of the Transmit Buffer */
        if(cecTransMsg.txCnt == (cecTxMsg.txMsgLen + 1))
        {
            /* Transmits message with an EOM bit*/
            CEC_CompleteMessage();
        
            CEC_TxData(cecTransMsg.buffer[cecTransMsg.txCnt++]);
        }
        else
        {
            /* Put the byte in the TX Buffer */
            CEC_TxData(cecTransMsg.buffer[cecTransMsg.txCnt++]);
        }
        
        CEC_ClearIntFlag(CEC_INT_TXBR);
    }
}

/*!
 * @brief       TMR14 init
 *
 * @param       None
 *
 * @retval      None
 *
 */
void APM_EVAL_TMR14_Init(uint16_t period, uint16_t div)
{
    TMR_TimeBase_T  timeBaseConfig;

    /* Enable Clock*/
    RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_SYSCFG);
    RCM_EnableAPB1PeriphClock(RCM_APB1_PERIPH_TMR14);

    /* Set clockDivision = 1 */
    timeBaseConfig.clockDivision =  TMR_CKD_DIV1;
    /* Up-counter */
    timeBaseConfig.counterMode =  TMR_COUNTER_MODE_UP;
    /* Set divider */
    timeBaseConfig.div = div - 1 ;
    /* Set counter = 0xffff */
    timeBaseConfig.period = period - 1;
    /* Repetition counter = 0x0 */
    timeBaseConfig.repetitionCounter =  0;

    TMR_ConfigTimeBase(TMR14,&timeBaseConfig);

    /* Enable update interrupt*/
    TMR_EnableInterrupt(TMR14,TMR_INT_UPDATE);
    NVIC_EnableIRQRequest(TMR14_IRQn,2);

    /*  Enable TMR14  */
    TMR_Enable(TMR14);
}

/*!
 * @brief       TMR4 interrupt service runtine
 *
 * @param       None
 *
 * @retval      None
 *
 * @note        This function need to put into TMR14_IRQHandler() in apm32f0xx_int.c
 */
void TMR14_Isr(void)
{
    if(TMR_ReadIntFlag(TMR14,TMR_INT_FLAG_UPDATE) == SET)
    {
        TMR_ClearIntFlag(TMR14,TMR_INT_FLAG_UPDATE);
        cntTick++;
        
        if(cntTick >= 500)
        {
            cntTick = 0;
            APM_EVAL_LEDToggle(LED1);
        }
    }
}

/*!
 * @brief       Redirect C Library function printf to serial port.
 *              After Redirection, you can use printf function.
 *
 * @param       ch:  The characters that need to be send.
 *
 * @param       *f:  pointer to a FILE that can recording all information
 *              needed to control a stream
 *
 * @retval      The characters that need to be send.
 *
 */
int fputc(int ch, FILE *f)
{
    /** send a byte of data to the serial port */
    USART_TxData(DEBUG_USART,(uint8_t)ch);

    /** wait for the data to be send  */
    while (USART_ReadStatusFlag(DEBUG_USART, USART_FLAG_TXBE) == RESET);

    return (ch);
}

/**@} end of group CEC_TwoBoardsExchange_Functions */
/**@} end of group CEC_TwoBoardsExchange */
/**@} end of group Examples */
