/*!
 * @file        main.c
 *
 * @brief       Main program body
 *
 * @version     V1.0.0
 *
 * @date        2022-09-29
 *
 * @attention
 *
 *  Copyright (C) 2022 Geehy Semiconductor
 *
 *  You may not use this file except in compliance with the
 *  GEEHY COPYRIGHT NOTICE (GEEHY SOFTWARE PACKAGE LICENSE).
 *
 *  The program is only for reference, which is distributed in the hope
 *  that it will be usefull and instructional for customers to develop
 *  their software. Unless required by applicable law or agreed to in
 *  writing, the program is distributed on an "AS IS" BASIS, WITHOUT
 *  ANY WARRANTY OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the GEEHY SOFTWARE PACKAGE LICENSE for the governing permissions
 *  and limitations under the License.
 */

/* Includes */
#include "main.h"
#include <stdio.h>
#include <string.h>
#include "bsp_lcd.h"
#include "bsp_key.h"
#include "tos_k.h"

/** @addtogroup Examples
  * @brief TencentOS Tiny examples
  @{
  */

/** @addtogroup TencentOS_Tiny
  @{
  */

/** @defgroup TencentOS_Tiny_Macros Macros
  @{
*/

#define TASK1_STK_SIZE          512
#define TASK2_STK_SIZE          512
#define TASK3_STK_SIZE          512
#define START_TASK_STK_SIZE     512
/* printf using USART1  */
#define DEBUG_USART             USART1

/**@} end of group TencentOS_Tiny_Macros*/

/** @defgroup TencentOS_Tiny_Variables Variables
  @{
  */

uint32_t voltage = 0;

k_task_t task1;
k_task_t task2;
k_task_t task3;
k_task_t taskStart;

uint8_t task1_stk[TASK1_STK_SIZE];
uint8_t task2_stk[TASK2_STK_SIZE];
uint8_t task3_stk[TASK3_STK_SIZE];
uint8_t start_task_stk[START_TASK_STK_SIZE];

/* Menu title string*/
const char *menuTitleStr[] = {
    " TencentOS Tiny Example ",
};

/* Menu item string*/
const char *menuItemStr[] = {
    "Press KEY1 to start test",
    "       Test OK          ",
    "       Test Error       ",
    "       Testing          ",
};

/* Menu bottom column string*/
const char *menuBottomColStr[] = {
    "   GEEHY SEMICONDUCTOR   ",
};

/**@} end of group TencentOS_Tiny_Variables*/

/** @defgroup TencentOS_Tiny_Functions Functions
  @{
  */

void Task1_Entry(void *arg);
void Task2_Entry(void *arg);
void Task3_Entry(void *arg);
void TaskStart_Entry(void *arg);

void APM_EVAL_Init(void);
void ADC1_Init(void);
void Menu_DisplayInit(void);

/*!
 * @brief       Main program
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
int main(void)
{
    APM_EVAL_Init();
    printf("This is an example of TencentOS Tiny(%s) \r\n",TOS_VERSION);
    
    tos_knl_init();
    tos_task_create(&taskStart, "start_task", TaskStart_Entry, NULL, 4, start_task_stk, START_TASK_STK_SIZE, 0);
    tos_knl_start();
}

/*!
 * @brief       Init EVAL board
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void APM_EVAL_Init(void)
{
    /* Init delay function */
    APM_EVAL_DelayInit();
    
    APM_EVAL_LEDInit(LED1);
    APM_EVAL_LEDInit(LED2);
    APM_EVAL_LEDInit(LED3);
    APM_EVAL_PBInit(BUTTON_KEY1,BUTTON_MODE_GPIO);
    APM_EVAL_COMInit(COM1);
    
    APM_LCDInit();
    LCD_Clear(0,0,LCD_WIDTH,LCD_HEIGHT,LCD_BCOLOR);
    
    Menu_DisplayInit();
    
    ADC1_Init();
    
    /* Wait until ADC is ready */
    while (!ADC_ReadStatusFlag(ADC_FLAG_ADRDY));

    ADC_StartConversion();
    
    APM_EVAL_LEDOff(LED2);
    APM_EVAL_LEDOff(LED3);
    LCD_Clear(0,LCD_LINE_2,LCD_WIDTH,134,LCD_BCOLOR);
}

/*!
 * @brief       Entry of start task
 *
 * @param       None
 *
 * @retval      None
 *
 */
void TaskStart_Entry(void *arg)
{
    tos_task_create(&task1, "task1", Task1_Entry, NULL, 3, task1_stk, TASK1_STK_SIZE, 0);
    tos_task_create(&task2, "task2", Task2_Entry, NULL, 3, task2_stk, TASK2_STK_SIZE, 0);
    tos_task_create(&task3, "task3", Task3_Entry, NULL, 3, task3_stk, TASK3_STK_SIZE, 0);
}

/*!
 * @brief       Entry of task1
 *
 * @param       None
 *
 * @retval      None
 *
 */
void Task1_Entry(void *arg)
{
    while(1)
    {
        LCD_DisplayIntNum(128,LCD_LINE_2,voltage,4,LCD_TEXT_FC,LCD_TEXT_BC,24);
        LCD_DisplayString(180,LCD_LINE_2,"mV",LCD_TEXT_FC,LCD_TEXT_BC,24,0);
        tos_task_delay(50);
    }
}

/*!
 * @brief       Entry of task2
 *
 * @param       None
 *
 * @retval      None
 *
 */
void Task2_Entry(void *arg)
{
    uint32_t adcData = 0;
    
    while(1)
    {
        /* Read ADC Conversion value*/
        adcData = ADC_ReadConversionValue();
        /* voltage(mV) =  adcData * (3300mV / 4095) */
        voltage = (adcData * 3300) / 4095;
        
        tos_task_delay(100);
    }
}

/*!
 * @brief       Entry of task3
 *
 * @param       None
 *
 * @retval      None
 *
 */
void Task3_Entry(void *arg)
{
    while(1)
    {
        printf("Hello world!\r\n");
        
        tos_task_delay(1000);
    }
}

/*!
 * @brief       Initialize the Menu UI
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void Menu_DisplayInit(void)
{
    /* Menu title*/
    LCD_Clear(0,0,LCD_WIDTH,40,LCD_FCOLOR);
    LCD_DisplayString(10,LCD_LINE_TITLE,menuTitleStr[0],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,24,0);
    
    /* Menu bottom*/
    LCD_Clear(0,LCD_LINE_BOTTOM,LCD_WIDTH,LCD_HEIGHT,LCD_FCOLOR);
    LCD_DisplayString(60,LCD_LINE_BOTTOM,menuBottomColStr[0],LCD_TEXT_SELECT_FC,LCD_TEXT_SELECT_BC,16,0);
}

/*!
 * @brief       ADC1 Init
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void ADC1_Init(void)
{
    GPIO_Config_T gpioConfig;
    ADC_Config_T  adcConfig;

    /* RCM Enable*/
    RCM_EnableAHBPeriphClock(RCM_AHB_PERIPH_GPIOA);
    RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_ADC1);
    RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_SYSCFG);
    /* GPIO Configuration */
    gpioConfig.pin = GPIO_PIN_1;
    gpioConfig.mode = GPIO_MODE_AN;
    gpioConfig.pupd = GPIO_PUPD_PU;
    GPIO_Config(GPIOB, &gpioConfig);
    /* ADC Configuration */
    ADC_Reset();
    ADC_ConfigStructInit(&adcConfig);
    /* Set resolution*/
    adcConfig.resolution = ADC_RESOLUTION_12B;
    /* Set dataAlign*/
    adcConfig.dataAlign  = ADC_DATA_ALIGN_RIGHT;
    /* Set scanDir*/
    adcConfig.scanDir    = ADC_SCAN_DIR_UPWARD;
    /* Set convMode continous*/
    adcConfig.convMode   = ADC_CONVERSION_CONTINUOUS;
    /* Set extTrigConv*/
    adcConfig.extTrigConv  = ADC_EXT_TRIG_CONV_TRG0;
    /* Set TrigEdge*/
    adcConfig.extTrigEdge    = ADC_EXT_TRIG_EDGE_NONE;

    ADC_Config(&adcConfig);
    ADC_ConfigChannel(ADC_CHANNEL_9, ADC_SAMPLE_TIME_239_5);

    NVIC_EnableIRQRequest(ADC1_COMP_IRQn, 2);

    /* Calibration*/
    ADC_ReadCalibrationFactor();
    /* Enable ADC*/
    ADC_Enable();
}

/*!
 * @brief       Redirect C Library function printf to serial port.
 *              After Redirection, you can use printf function.
 *
 * @param       ch:  The characters that need to be send.
 *
 * @param       *f:  pointer to a FILE that can recording all information
 *              needed to control a stream
 *
 * @retval      The characters that need to be send.
 */
int fputc(int ch, FILE* f)
{
    /* send a byte of data to the serial port */
    USART_TxData(DEBUG_USART, (uint8_t)ch);

    /* wait for the data to be send  */
    while (USART_ReadStatusFlag(DEBUG_USART, USART_FLAG_TXBE) == RESET);

    return (ch);
}

/**@} end of group TencentOS_Tiny_Functions */
/**@} end of group TencentOS_Tiny */
/**@} end of group Examples */
